#!/bin/sh
#
# author: Joerg Heinemann
# website: http://enterprise-log-management-appliance.org
# license: GPL v3
#
# /usr/share/elma-devel/bin/elma-build.sh
#

RETVAL=0
LOGGING=0
#IFS="_"
OPTIONS=$@

PACKAGESDIR="/opt/packages"
RSTDIR="$PACKAGESDIR/elma-doc"
DOCDIR="/usr/share/doc/packages/elma"
KIBANA_SOURCEDIR="$PACKAGESDIR/elasticsearch-kibana"
KIBANA_RSTDIR="$PACKAGESDIR/elasticsearch-kibana-doc"
KIBANA_DOCDIR="/usr/share/doc/packages/elasticsearch-kibana"
ELASTICSEARCH_SOURCEDIR="$PACKAGESDIR/elasticsearch"
ELASTICSEARCH_RSTDIR="$PACKAGESDIR/elasticsearch-doc"
ELASTICSEARCH_DOCDIR="/usr/share/doc/packages/elasticsearch"
RSYSLOG_RSTDIR="$PACKAGESDIR/rsyslog-doc"
RSYSLOG_DOCDIR="/usr/share/doc/packages/rsyslog"

SCRIPT_DESCRIPTION="ELMA - Package builds and documentation updates"
SCRIPT_AUTHOR="Joerg Heinemann"

# ######### Load ELMA configuration parameter, global functions and script header #########
#
source /etc/elma/elma.conf ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
source $ELMA_LIB_DIR/elma-global-functions.sh ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
source $ELMA_DEVEL_LIB_DIR/elma-devel-global-functions.sh ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

# ######### Checking command line options #########
#
if [ -z "$*" ]; then
	script_logging ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	script_header $OPTIONS ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	build_elma_package ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
fi

while test -n "$1"; do
	case "$1" in
	''|"")
		if [ $LOGGING -eq 0 ]; then
			script_logging ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
			script_header $OPTIONS ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
			LOGGING=1
		fi
		build_elma_package ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		shift
		;;
	--elma)
		if [ $LOGGING -eq 0 ]; then
			script_logging ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
			script_header $OPTIONS ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
			LOGGING=1
		fi
		build_elma_package ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		shift
		;;
	--elma-doc)
		if [ $LOGGING -eq 0 ]; then
			script_logging ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
			script_header $OPTIONS ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
			LOGGING=1
		fi
		build_elmadoc_package ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		shift
		;;
	--elma-doc-build)
		if [ $LOGGING -eq 0 ]; then
			script_logging ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
			script_header $OPTIONS ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
			LOGGING=1
		fi
		build_sphinx_html $RSTDIR $DOCDIR ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		shift
		;;
	--elasticsearch-doc)
		if [ $LOGGING -eq 0 ]; then
			script_logging ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
			script_header $OPTIONS ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
			LOGGING=1
		fi
		build_elasticsearchdoc_package ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		shift
		;;
	--elasticsearch-kibana-doc)
		if [ $LOGGING -eq 0 ]; then
			script_logging ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
			script_header $OPTIONS ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
			LOGGING=1
		fi
		build_kibanadoc_package ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		shift
		;;
	--rsyslog-doc)
		if [ $LOGGING -eq 0 ]; then
			script_logging ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
			script_header $OPTIONS ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
			LOGGING=1
		fi
		build_rsyslogdoc_package ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		shift
		;;
	*)
		echo
		echo "$(basename $0): invalid option -- '$1'" >&2
		echo "$(basename $0): Usage: $(basename $0) <options>" >&2
		echo
		echo "Options:"
		echo
		echo "--help                       Display this help message and exit"
		echo
		echo "--elma                       Creation of a new ELMA package"
		echo "--elma-doc                   Creation of a new ELMA documentation package"
		echo "--elma-doc-build             Build of new ELMA html documentation pages"
		echo "--elasticsearch-doc          Creation of a new elasticsearch documentation package"
		echo "--elasticsearch-kibana-doc   Creation of a new Kibana documentation package"
		echo "--rsyslog-doc                Creation of a new rsyslog documentation package"
		echo
		exit 1
		;;
	esac
done

# ######### Final checks #########
#
if [ $RETVAL -eq 0 ]
then
	echo "==========================================================================================="
	echo "PASS: Successful"
	echo "==========================================================================================="
	echo
else
	echo "ABORT - Please check above error messages and installation log file!"
	echo
	exit 1
fi

# ######### Time to close output and error named pipes #########
#
sleep 1