#!/bin/sh
#
# author: Joerg Heinemann
# website: http://enterprise-log-management-appliance.org
# license: GPL v3
#
# /usr/share/elma-devel/bin/rsyslog-compile.sh
#

RETVAL=0

SCRIPT_DESCRIPTION="Build and deployment of a new rsyslog package"
SCRIPT_AUTHOR="Joerg Heinemann"

# Newest stable rsyslog version
#
NEWEST_STABLE_RSYSLOG_VERSION="rsyslog-7.4.4"

# ######### rsyslog sources download and install parameter #########
#
PACKAGEFILE="$NEWEST_STABLE_RSYSLOG_VERSION.tar.gz"
PACKAGEURL="http://www.rsyslog.com/files/download/rsyslog/$PACKAGEFILE"
PACKAGENAME="rsyslog-7"
LINKNAME="rsyslog"

# ######### Load ELMA configuration parameter, global functions and script header #########
#
source /etc/elma/elma.conf ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
source $ELMA_LIB_DIR/elma-global-functions.sh ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
source $ELMA_DEVEL_LIB_DIR/elma-devel-global-functions.sh ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
script_logging ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
script_header ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

# ######### Check .netrc and .gitconfig user configuration for git client #########
#
check_netrc_config ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
check_git_config ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

# ######### Init local git repositories #########
#
#init_git $ELMA_DIR $NEW_ELMA_DIR ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

# ######### Install and update required openSUSE packages #########
#
#source $ELMA_DIR/bin/install/suse-update.sh ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

# ######### Install required openSUSE development packages #########
#
install_opensuse_devel_packages ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

# ######### Install and update rsyslog sources #########
#
echo "--- `date +%T.%N | cut -c 1-12` - Install and update rsyslog sources -------------------------------------"
echo

download_new_package $PACKAGEURL $INSTALLDIR $PACKAGEFILE ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
untar_new_package $PACKAGENAME $INSTALLDIR ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
link_new_package $PACKAGENAME $INSTALLDIR $LINKNAME ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

NEW_RSYSLOG_VERSION=$(cd $INSTALLDIR; find -name "$PACKAGENAME*" -type d -exec stat -t {} \; | sort -r -n -k 13,13 | head --lines=1 | awk -F' ' '{print $1}' | sed 's/\ .*$//;s/.\///;s/rsyslog-//')
NEW_RSYSLOG_DIR="$INSTALLDIR/rsyslog-$NEW_RSYSLOG_VERSION"
NEW_RSYSLOG_TESTTOOLS_DIR="$INSTALLDIR/rsyslog-testtools-$NEW_RSYSLOG_VERSION"

# ######### Compiling rsyslog #########
#
echo "--- `date +%T.%N | cut -c 1-12` - Compiling rsyslog ------------------------------------------------------"
echo

cd $NEW_RSYSLOG_DIR; ./configure --enable-regexp --enable-zlib --enable-pthreads --enable-klog --enable-inet --enable-unlimited-select --enable-rtinst --enable-debugless --enable-memcheck --enable-diagtools --enable-mysql --enable-snmp --enable-rsyslogrt --enable-rsyslogd --enable-extended-tests --enable-mail --enable-imptcp --enable-omruleset --enable-valgrind --enable-imdiag --enable-relp --enable-testbench --enable-imfile --enable-omstdout --enable-omuxsock --enable-pmlastmsg --enable-omudpspoof --enable-omprog --enable-impstats --enable-mysql-tests --enable-usertools --enable-libdbi --enable-imttcp --enable-elasticsearch --enable-sm_cust_bindcdr --enable-mmnormalize --enable-mmjsonparse --enable-mmaudit --enable-mmsnmptrapd --enable-pmcisconames --enable-pmaixforwardedfrom --enable-pmsnare --enable-pmrfc3164sd --enable-kmsg --enable-gnutls --enable-gssapi-krb5
### --enable-omhdfs --enable-imsolaris --enable-debug   
cd $NEW_RSYSLOG_DIR; make

export srcdir=$NEW_RSYSLOG_DIR/tests
cd $NEW_RSYSLOG_DIR/tests; make check
echo

# ######### Copy rsyslog testtools #########
#
#echo "--- `date +%T.%N | cut -c 1-12` - Copy rsyslog testtools -------------------------------------------------"
#echo
#
#if [ ! -d $NEW_RSYSLOG_TESTTOOLS_DIR ]
#then
#	mkdir $NEW_RSYSLOG_TESTTOOLS_DIR
#fi
#
#cp $NEW_RSYSLOG_DIR/AUTHORS $NEW_RSYSLOG_TESTTOOLS_DIR
#cp $NEW_RSYSLOG_DIR/ChangeLog $NEW_RSYSLOG_TESTTOOLS_DIR
#cp $NEW_RSYSLOG_DIR/COPYING $NEW_RSYSLOG_TESTTOOLS_DIR
#cp $NEW_RSYSLOG_DIR/COPYING.ASL20 $NEW_RSYSLOG_TESTTOOLS_DIR
#cp $NEW_RSYSLOG_DIR/COPYING.LESSER $NEW_RSYSLOG_TESTTOOLS_DIR
#cp $NEW_RSYSLOG_DIR/README $NEW_RSYSLOG_TESTTOOLS_DIR
#
#cp $NEW_RSYSLOG_DIR/tests/chkseq $NEW_RSYSLOG_TESTTOOLS_DIR
#cp $NEW_RSYSLOG_DIR/tests/diagtalker $NEW_RSYSLOG_TESTTOOLS_DIR
#cp $NEW_RSYSLOG_DIR/tests/inputfilegen $NEW_RSYSLOG_TESTTOOLS_DIR
#cp $NEW_RSYSLOG_DIR/tests/minitcpsrv $NEW_RSYSLOG_TESTTOOLS_DIR
#cp $NEW_RSYSLOG_DIR/tests/msleep $NEW_RSYSLOG_TESTTOOLS_DIR
#cp $NEW_RSYSLOG_DIR/tests/nettester $NEW_RSYSLOG_TESTTOOLS_DIR
#cp $NEW_RSYSLOG_DIR/tests/ourtail $NEW_RSYSLOG_TESTTOOLS_DIR
#cp $NEW_RSYSLOG_DIR/tests/randomgen $NEW_RSYSLOG_TESTTOOLS_DIR
#cp $NEW_RSYSLOG_DIR/tests/syslog_caller $NEW_RSYSLOG_TESTTOOLS_DIR
#cp $NEW_RSYSLOG_DIR/tests/syslog_inject $NEW_RSYSLOG_TESTTOOLS_DIR
#cp $NEW_RSYSLOG_DIR/tests/tcpflood $NEW_RSYSLOG_TESTTOOLS_DIR
#cp $NEW_RSYSLOG_DIR/tests/uxsockrcvr $NEW_RSYSLOG_TESTTOOLS_DIR

# ######### Update rsyslog version variable #########
#
#echo "--- `date +%T.%N | cut -c 1-12` - Update rsyslog version variable ----------------------------------------"
#echo
#
#sed -i -e 's/RSYSLOG_VERSION="'$RSYSLOG_VERSION'"/RSYSLOG_VERSION="'$NEW_RSYSLOG_VERSION'"/' $ELMA_DIR/config/elma/elma.conf ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
#sed -i -e 's/RSYSLOG_VERSION="'$RSYSLOG_VERSION'"/RSYSLOG_VERSION="'$NEW_RSYSLOG_VERSION'"/' $ELMA_CONF ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
#grep "RSYSLOG_VERSION=" $ELMA_CONF

# ######### Make rsyslog testtools tar gzip archiv #########
#
#echo "--- `date +%T.%N | cut -c 1-12` - Make rsyslog testtools tar gzip archiv ---------------------------------"
#echo
#
#cd $INSTALLDIR ; tar -cvzf rsyslog-testtools-$NEW_RSYSLOG_VERSION.tgz rsyslog-testtools-$NEW_RSYSLOG_VERSION ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
#echo

# ######### Upload new rsyslog testtools gzip archiv to Google Code downloads #########
#
#RELEASE_NAME="rsyslog-testtools"
#RELEASE_TRAIN="v7-stable"
#UPLOAD_SUMMARY="rsyslog Tools $NEW_RSYSLOG_VERSION"
#UPLOAD_DESCRIPTION="Rsyslog comes with a couple of tools. Most of them have originally been developed for other purposes and are only included in source distributions. This packuage is compiled with openSuse 12.2 for using the rsyslog traffic generator (tcpflood)."
#UPLOAD_FILE="rsyslog-testtools-$NEW_RSYSLOG_VERSION.tgz"
#
#upload_new_package ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

# ######### Modify wiki changelog #########
#
#GIT_ORIGIN="master"
#
#modify_wiki_changelog $ELMA_DIR $NEW_RSYSLOG_VERSION

# ######### Final checks of install and update process #########
#
if [ $RETVAL -eq 0 ]
then
	echo "==========================================================================================="
	echo "PASS: Build and deployment of new rsyslog package was successful"
	echo "-------------------------------------------------------------------------------------------"
	echo
	echo "New Version:	$NEW_RSYSLOG_VERSION"
	echo
	echo "==========================================================================================="
	echo
else
	echo "ABORT - Build and deployment of new ELMA package was NOT successful!"
	echo "Please check above error messages and installation log file."
	echo
fi

# ######### Time to close output and error named pipes #########
#
sleep 1
