#!/bin/sh
#
# author: Joerg Heinemann
# website: http://enterprise-log-management-appliance.org
# license: GPL v3
#
# /usr/share/elma-devel/lib/elma-devel-global-functions.sh
#


# #########################################################################################
#
# Development functions
#
# #########################################################################################


# ######### Check .netrc user configuration for git client #########
#
check_netrc_config(){

	local RETVAL=0

	if [ ! -f ~/.netrc ]
	then
		echo "--- `date +%T.%N | cut -c 1-12` - Make .netrc user configuration for git client --------------------------"
		echo

		cp $ELMA_DEVEL_DIR/config/.netrc ~/.netrc ; if [ $? -ne 0 ]; then error_exit "$LINENO: An error has occurred."; fi
		chmod 600 ~/.netrc ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

		echo "You have to check the right file permissions for your ~/.netrc password file!!!"
		echo "chmod 600 ~/.netrc" 
		echo
		echo "Set the right github.com project member UID and Password in your ~/.netrc password file:"
		echo
		cat ~/.netrc; echo
		echo
		RETVAL=1
	else
		RETVAL=0
	fi

	return $RETVAL

}

# ######### Check ~/.gitconfig user configuration for git client #########
#
check_git_config(){

	local RETVAL=0

	if [ ! -f ~/.gitconfig ]
	then
		echo "--- `date +%T.%N | cut -c 1-12` - Make ~/.gitconfig user configuration for git client --------------------"
		echo

		git config --global user.name "your-github-member-UID" ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		git config --global user.email your-github-member-UID@gmail.com ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		chmod 600 ~/.gitconfig ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		echo
		RETVAL=1
	else
		RETVAL=0
	fi

	return $RETVAL

}

# ######### Init local git repositories #########
#
# NOT in use
#
init_git(){

	local RETVAL=0
	local DIR="$1"
	local NEWDIR="$2"

	if [ ! $DIR = $NEWDIR ]
	then

		if [ ! -d $DIR/.git ]
		then
			echo "--- `date +%T.%N | cut -c 1-12` - Init local git repository ----------------------------------------------"
			echo

			cd $INSTALLDIR; env GIT_SSL_NO_VERIFY=true git clone https://code.google.com/p/enterprise-log-management-appliance.master/ $NEWDIR ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
			echo
		fi

		if [ ! -d $DIR/doc/wiki/.git ]
		then
			echo "--- `date +%T.%N | cut -c 1-12` - Init local wiki git repository -----------------------------------------"
			echo

			cd $NEWDIR/doc/wiki; git init ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
			echo
			cd $NEWDIR/doc/wiki; env GIT_SSL_NO_VERIFY=true git remote add origin https://code.google.com/p/enterprise-log-management-appliance.wiki ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		fi

		RETVAL=0
	else
		RETVAL=1
	fi

	return $RETVAL

}

# ######### Pull new revision from remote git repository #########
#
pull_git(){

	local RETVAL=0
	local DIR=$1

	echo "--- `date +%T.%N | cut -c 1-12` - Pull new $GIT_ORIGIN revision from remote git repository ---------------"
	echo

#	cd $DIR; env GIT_SSL_NO_VERIFY=true git pull origin $GIT_ORIGIN ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	cd $DIR; env GIT_SSL_NO_VERIFY=true git pull origin $GIT_ORIGIN
	echo

	return $RETVAL

}

# ######### Push new revision to remote git repository #########
#
push_git(){

	local RETVAL=0
	local DIR="$1"
	local GIT_CHANGELOG="$2"

	echo "--- `date +%T.%N | cut -c 1-12` - Push new $GIT_ORIGIN revision to remote git repository -----------------"
	echo

	cd $DIR; git add . ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

	# Add additional changelog to git commit message
	#
	if [ $DIR = $NEW_ELMA_DIR ]
	then
		echo $GIT_CHANGELOG > $DIR/.git/COMMIT_EDITMSG ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		grep -E '^ * ' $DIR/CHANGELOG >> $DIR/.git/COMMIT_EDITMSG ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
#		cd $DIR; git commit -aF $DIR/.git/COMMIT_EDITMSG ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		cd $DIR; git commit -aF $DIR/.git/COMMIT_EDITMSG
	else
		cd $DIR; git commit -m "$GIT_CHANGELOG" ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	fi

	cat $DIR/.git/COMMIT_EDITMSG ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	echo

#	cd $DIR; env GIT_SSL_NO_VERIFY=true git push origin $GIT_ORIGIN ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	cd $DIR; env GIT_SSL_NO_VERIFY=true git push origin $GIT_ORIGIN -f
	echo

	return $RETVAL

}

# ######### Modify wiki changelog #########
#
modify_wiki_changelog(){

	local RETVAL=0

	local GIT_CHANGELOG="Modify changelog to elma $NEW_ELMA_VERSION"
	local WIKI_CHANGELOG_HEADER="====elma (master) $NEW_ELMA_VERSION - `date +'%Y-%m-%d'`===="

	# Modify local wiki changelog
	#
	echo "--- `date +%T.%N | cut -c 1-12` - Modify local wiki changelog --------------------------------------------"
	echo

	if [ "$NEW_ELMA_VERSION" ]
	then
		sed -i -e '/^.*elma (master) - /s/elma (master) - .*/elma (master) - '$NEW_ELMA_VERSION' ('`date +'%Y-%m-%d'`')/' $WIKIDIR/Changelog.wiki ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	fi
	if [ "$ELMA_APPLIANCE_VERSION" ] && [ "$ELMA_APPLIANCE_VERSION_DATE" ]
	then
		sed -i -e '/^.*Enterprise_Log_Management_Appliance/s/Enterprise_Log_Management_Appliance.*/'$ELMA_APPLIANCE_VERSION' ('$ELMA_APPLIANCE_VERSION_DATE')/' $WIKIDIR/Changelog.wiki ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	fi
	if [ "$OPENSUSE_VERSION" ] && [ "$OPENSUSE_VERSION_DATE" ]
	then
		sed -i -e '/^.*Based on /s/Based on .*/Based on '"$OPENSUSE_VERSION"' ('$OPENSUSE_VERSION_DATE') <\/pre>/' $WIKIDIR/Changelog.wiki ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	fi
	if [ "$RSYSLOG_VERSION" ] && [ "$RSYSLOG_VERSION_DATE" ]
	then
		sed -i -e '/^.*rsyslog (v7-stable) - /s/rsyslog (v7-stable) - .*/rsyslog (v7-stable) - '$RSYSLOG_VERSION' ('$RSYSLOG_VERSION_DATE')/' $WIKIDIR/Changelog.wiki ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	fi
	if [ "$RSYSLOG_TESTTOOLS_VERSION" ] && [ "$RSYSLOG_TESTTOOLS_VERSION_DATE" ]
	then
		sed -i -e '/^.*rsyslog-testtools (v7-stable) - /s/rsyslog-testtools (v7-stable) - .*/rsyslog-testtools (v7-stable) - '$RSYSLOG_TESTTOOLS_VERSION' ('$RSYSLOG_TESTTOOLS_VERSION_DATE')/' $WIKIDIR/Changelog.wiki ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	fi
	if [ "$NAGIOS_CORE_VERSION" ] && [ "$NAGIOS_CORE_VERSION_DATE" ]
	then
		sed -i -e '/^.*nagios core (3.x) - /s/nagios core (3.x) - .*/nagios core (3.x) - '$NAGIOS_CORE_VERSION' ('$NAGIOS_CORE_VERSION_DATE')/' $WIKIDIR/Changelog.wiki ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	fi
	#if [ "$APACHE2_VERSION" ] && [ "$APACHE2_VERSION_DATE" ]
	#then
	#	sed -i -e '/^.*apache (2.2-stable) - /s/apache (2.2-stable) - .*/apache (2.2-stable) - '$APACHE2_VERSION' ('$APACHE2_VERSION_DATE')/' $WIKIDIR/Changelog.wiki ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	#fi
	if [ "$LOGSTASH_VERSION" ] && [ "$LOGSTASH_VERSION_DATE" ]
	then
		sed -i -e '/^.*logstash (monolithic.jar) - /s/logstash (monolithic.jar) - .*/logstash (monolithic.jar) - '$LOGSTASH_VERSION' ('$LOGSTASH_VERSION_DATE')/' $WIKIDIR/Changelog.wiki ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	fi
	if [ "$ES_VERSION" ] && [ "$ES_VERSION_DATE" ]
	then
		sed -i -e '/^.*elasticsearch (stable) - /s/elasticsearch (stable) - .*/elasticsearch (stable) - '$ES_VERSION' ('$ES_VERSION_DATE')/' $WIKIDIR/Changelog.wiki ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	fi
	if [ "$KIBANA3_VERSION" ] && [ "$KIBANA3_VERSION_DATE" ]
	then
		sed -i -e '/^.*elasticsearch-kibana3 (master) - /s/elasticsearch-kibana3 (master) - .*/elasticsearch-kibana3 (master) - '$KIBANA3_VERSION' ('$KIBANA3_VERSION_DATE')/' $WIKIDIR/Changelog.wiki ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	fi

	echo -e "$WIKI_CHANGELOG_HEADER" >> $ELMA_DIR/CHANGELOG.tmp ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	grep -E '^ * ' $ELMA_DIR/CHANGELOG >> $ELMA_DIR/CHANGELOG.tmp ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	sed -i '32r '$ELMA_DIR/CHANGELOG.tmp'' $WIKIDIR/Changelog.wiki ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	echo -e "\n" >> $ELMA_DIR/CHANGELOG.tmp ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

	rm $ELMA_DIR/CHANGELOG.tmp ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	echo

	# ######### Upload new wiki revision to remote git repository #########
	#
	echo "--- `date +%T.%N | cut -c 1-12` - Push new wiki revision to remote git repository ------------------------"
	echo

	cd $WIKIDIR; git add . ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

	# Add additional changelog to git commit message
	#
	cd $WIKIDIR; git commit -m "$GIT_CHANGELOG" ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

	cat $WIKIDIR/.git/COMMIT_EDITMSG ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	echo

	cd $WIKIDIR; env GIT_SSL_NO_VERIFY=true git push ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	echo

	return $RETVAL

}

# ######### Convert documentation from asciidoc to rst #########
#
convert_asciidoc_to_rst(){

	local RETVAL=0
	local ASCIIDOCDIR=$1

	echo "--- `date +%T.%N | cut -c 1-12` - Convert documentation from asciidoc to rst -----------------------------"
	echo
	cd $ASCIIDOCDIR ; asciidoc --backend docbook --doctype book --attribute icons --attribute docinfo1 --attribute tabsize=4 --out-file index.xml index.asciidoc
	cd $ASCIIDOCDIR ; pandoc -s -S index.xml -f docbook -w rst -o index.rst ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	echo

	return $RETVAL

}

# ######### Create new gzip archiv #########
#
create_new_archive(){

	local RETVAL=0
	local ROOTDIR=$1
	local TARDIR=$2

	echo "--- `date +%T.%N | cut -c 1-12` - Create new tar gzip archiv ---------------------------------------------"
	echo

	if [ ! -d $ELMA_FTP_DIR ]
	then
		mkdir $ELMA_FTP_DIR ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	fi

	if [ ! -d $ELMA_FTP_DIR/downloads ]
	then
		mkdir $ELMA_FTP_DIR/downloads ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	fi

#	cd $PACKAGESDIR ; tar -cvzf elma/$PACKAGENAME.tgz --exclude .git --exclude .gitignore $PACKAGENAME ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	cd $PACKAGESDIR ; tar -cvzf $ELMA_FTP_DIR/downloads/$PACKAGENAME.tgz --exclude .git $PACKAGENAME ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	echo

	return $RETVAL

}

# ######### Install required openSUSE development packages #########
#
install_opensuse_devel_packages(){

	local RETVAL=0

	echo "--- `date +%T.%N | cut -c 1-12` - Install required openSUSE development packages -------------------------"
	echo
	zypper -qn --no-gpg-checks --gpg-auto-import-keys install gcc make autoconf flex bison libestr-devel libee-devel libjson-devel zlib-devel librelp-devel libmysqld-devel net-snmp-devel libgnutls-devel libnet-devel libdbi-devel libuuid-devel libcurl-devel liblognorm-devel libgcrypt-devel krb5-devel valgrind-devel python-docutils ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	echo

	return $RETVAL

}

# ######### Delete temp directory #########
#
delete_temp_devel_directory(){

	local RETVAL=0
	local PACKAGESDIR=$1
	local PACKAGENAME=$2

	echo "--- `date +%T.%N | cut -c 1-12` - Delete temp directory --------------------------------------------------"
	echo

	if [ -d $PACKAGESDIR/$PACKAGENAME ]
	then
		rm -R $PACKAGESDIR/$PACKAGENAME ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	fi

	return $RETVAL

}

# ######### Creation of a new ELMA package #########
#
build_elma_package(){

	local RETVAL=0

	# ######### ELMA-WWW download and install parameter #########
	#

	NEW_ELMA_DATE=`date '+%Y-%m-%d'`
	OLD_ELMA_VERSION=$(echo $ELMA_VERSION | awk -F'.' '{print $1 "." $2 "." $3+0}')

	read -e -p "New version: " -i $OLD_ELMA_VERSION NEW_ELMA_VERSION
	echo

	local PACKAGENAME="elma-$NEW_ELMA_VERSION"
	WWWDIR="$PACKAGESDIR/$PACKAGENAME/elma-www"

	echo "--- `date +%T.%N | cut -c 1-12` - Creation of a new ELMA package -----------------------------------------"
	echo

	# ######### Check .netrc and .gitconfig user configuration for git client #########
	#
	check_netrc_config ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	check_git_config ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

	# ######### Delete temp directory #########
	#
	delete_temp_devel_directory $PACKAGESDIR $PACKAGENAME ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

	# ######### Create package directories #########
	#
	if [ ! -d $PACKAGESDIR ]
	then
		mkdir $PACKAGESDIR ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	fi
	if [ ! -d $PACKAGESDIR/$PACKAGENAME ]
	then
		mkdir $PACKAGESDIR/$PACKAGENAME ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	fi
	if [ ! -d $WWWDIR ]
	then
		mkdir $WWWDIR ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	fi
	echo

	# ######### Set permissions for ELMA web directory #########
	#
	reconfigure_elma_web_directory ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

	# ######### Copy required files into new directory #########
	#
	echo "--- `date +%T.%N | cut -c 1-12` - Copy required files into new tmp directory -----------------------------"
	echo

	cp -R $ELMA_WWW_DIR/* $WWWDIR ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	cp -R $ELMA_WWW_DIR/.htaccess $WWWDIR ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	echo

	# ######### Delete work files #########
	#
	echo "--- `date +%T.%N | cut -c 1-12` - Delete work files ------------------------------------------------------"
	echo

	rm -R $WWWDIR/data/_backup/pages/* ; cp $WWWDIR/data/_backup/index.html $WWWDIR/data/_backup/pages
	rm -R $WWWDIR/data/_cache/*
	rm -R $WWWDIR/data/_elthumbs/*
	rm -R $WWWDIR/data/_exports/*.gz
	rm -R $WWWDIR/data/_sessions/*.php
	rm -R $WWWDIR/data/_uploaded/downloads/*
	echo

	# ######### Create new gzip archiv #########
	#
	create_new_archive $PACKAGESDIR $PACKAGENAME ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

	# ######### Creation of a new ELMA documentation package #########
	#
	build_elmadoc_package ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

	# ######### git push and pull parameter #########
	#
#	GIT_ORIGIN="master"
#	GIT_CHANGELOG="Initial commit for elma-$NEW_ELMA_VERSION"
	
#	RELEASE_NAME="elma"
#	RELEASE_TRAIN="master"
#	UPLOAD_SUMMARY="ELMA $NEW_ELMA_VERSION"
#	UPLOAD_DESCRIPTION="Enterprise Log Management Appliance - Installation and update packet"
#	UPLOAD_FILE="$PACKAGESDIR/elma/$PACKAGENAME.tgz"
	
	# ######### Push new ELMA master revision to remote git repository #########
	#
#	push_git $NEW_ELMA_DIR "$GIT_CHANGELOG" ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	
	# ######### Delete temp directory #########
	#
	delete_temp_devel_directory $PACKAGESDIR $PACKAGENAME ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

	# ######### Final checks of install and update process #########
	#
	echo "-------------------------------------------------------------------------------------------"
	echo
	echo "Old Version:	$ELMA_VERSION"
	echo "New Version:	$NEW_ELMA_VERSION"
	echo

	return $RETVAL

}

# ######### Creation of a new ELMA documentation package #########
#
build_elmadoc_package(){

	local RETVAL=0

	if [ -z "$NEW_ELMA_VERSION" ]
	then
		local PACKAGENAME="elma-doc-$(echo $ELMA_VERSION | awk -F'.' '{print $1 "." $2 "." $3+0}')"
	else
		local PACKAGENAME="elma-doc-$NEW_ELMA_VERSION"
	fi

	echo "--- `date +%T.%N | cut -c 1-12` - Creation of a new ELMA documentation packages --------------------------"
	echo

	# ######### Check .netrc and .gitconfig user configuration for git client #########
	#
	check_netrc_config ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	check_git_config ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

	# ######### Delete temp directory #########
	#
	delete_temp_devel_directory $PACKAGESDIR $PACKAGENAME ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

	# ######### Create package directories #########
	#
	if [ ! -d $PACKAGESDIR ]
	then
		mkdir $PACKAGESDIR ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	fi
	if [ ! -d $PACKAGESDIR/$PACKAGENAME ]
	then
		mkdir $PACKAGESDIR/$PACKAGENAME ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	fi
	if [ ! -d $RSTDIR ]
	then
		mkdir $RSTDIR ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	fi
	echo

	# ######### Delete documentation build directories #########
	#
	rm -rf $RSTDIR/build
	echo 

	# ######### Download git documentation repository #########
	#
	if [ ! -d $RSTDIR/.git ]
	then
		# ######### Clone git documentation repository #########
		#
		echo "--- `date +%T.%N | cut -c 1-12` - Clone git documentation repository -------------------------------------"
		echo
		cd $PACKAGESDIR; env GIT_SSL_NO_VERIFY=true git clone https://github.com/enterprise-log-management-appliance/elma-doc.git/ $RSTDIR ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		echo
	else
		# Pull new revision from remote git documentation repository
		#
		echo "--- `date +%T.%N | cut -c 1-12` - Pull new revision from remote git documentation repository -------------"
		echo
		cd $RSTDIR; env GIT_SSL_NO_VERIFY=true git pull ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		echo
	fi

	# ######### sphinx build html doc pages #########
	#
	build_sphinx_html $RSTDIR $DOCDIR ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

	# ######### Build sitemap xml pages #########
	#
	build_sitemap_xml ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

	# ######### Copy required files into new directory #########
	#
	echo "--- `date +%T.%N | cut -c 1-12` - Copy required files into new tmp directory -----------------------------"
	echo

	cp -R $RSTDIR/* $PACKAGESDIR/$PACKAGENAME ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	echo

	# ######### Create new gzip archiv #########
	#
	create_new_archive $PACKAGESDIR $PACKAGENAME ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

	# ######### Delete temp directory #########
	#
	delete_temp_devel_directory $PACKAGESDIR $PACKAGENAME ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

	return $RETVAL

}

# ######### sphinx build html doc pages #########
#
build_sphinx_html(){

	local RETVAL=0
	local RSTDIR=$1
	local DOCDIR=$2

	# install/upgrade sphinx bootstrap theme
	#
	echo "--- `date +%T.%N | cut -c 1-12` - install/upgrade sphinx bootstrap theme ---------------------------------"
	echo
	pip install --upgrade sphinx_bootstrap_theme
	echo

	# sphinx build html doc pages
	#
	echo "--- `date +%T.%N | cut -c 1-12` - sphinx build html doc pages --------------------------------------------"
	echo

	cp $ELMA_DEVEL_DIR/sphinx/Makefile $RSTDIR ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	cp -R $ELMA_DEVEL_DIR/sphinx/* $PACKAGESDIR ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	rm $PACKAGESDIR/Makefile ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

#	cd $RSTDIR; sphinx-build -b html source build ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	cd $RSTDIR; make html ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	echo

	# ######### Copy html documentation into into ELMA doc directory #########
	#
	echo "--- `date +%T.%N | cut -c 1-12` - Copy html doc pages into into ELMA doc directory -----------------------"
	echo

	cp -R $RSTDIR/build/html $DOCDIR
	echo

	return $RETVAL

}

# ######### Build sitemap xml pages #########
#
build_sitemap_xml(){

	local RETVAL=0

	# Crawling project website
	#
	echo "--- `date +%T.%N | cut -c 1-12` - Crawling project website -----------------------------------------------"
	echo

	if [ ! -d $ELMA_DEVEL_DIR/sitemap_gen ]
	then
		mkdir $ELMA_DEVEL_DIR/sitemap_gen ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	fi
	echo

	wget -mk --spider -r -l4 http://enterprise-log-management-appliance.org/ -o $ELMA_DEVEL_DIR/sitemap_gen/urlinfolist.txt
	cat $ELMA_DEVEL_DIR/sitemap_gen/urlinfolist.txt | tr ' ' '\012' | grep "^http" | egrep -vi "[?]|[.]jpg$" | sort -u > $ELMA_DEVEL_DIR/sitemap_gen/urllist.txt ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	echo

	# Generate project sitemap
	#
	echo "--- `date +%T.%N | cut -c 1-12` - Generate project sitemap -----------------------------------------------"
	echo
 	PYTHON_SITE_PACKAGES_DIR=$(python -c 'from distutils.sysconfig import get_python_lib; print(get_python_lib())')
	python $PYTHON_SITE_PACKAGES_DIR/sitemap_gen.py --config=$ELMA_DEVEL_DIR/config/sitemap_gen-config-elma-doc.xml ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	echo

	rm $ELMA_DEVEL_DIR/sitemap_gen/urlinfolist.txt ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	rm $ELMA_DEVEL_DIR/sitemap_gen/urllist.txt ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

	return $RETVAL

}


# ######### Creation of a new Kibana documentation package #########
#
build_kibanadoc_package(){

	local RETVAL=0
	local OLDVERSION=$(check_kibana3_status; echo $KIBANA3_VERSION | awk -F'.' '{print $1 "." $2 "." $3+0}')

	read -e -p "New version: " -i $OLDVERSION NEWVERSION
	echo

	local PACKAGENAME="elasticsearch-kibana-doc-$NEWVERSION"

	echo "--- `date +%T.%N | cut -c 1-12` - Creation of a new Kibana documentation packages ------------------------"
	echo

	# ######### Check .netrc and .gitconfig user configuration for git client #########
	#
	check_netrc_config ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	check_git_config ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

	# ######### Delete temp directory #########
	#
	delete_temp_devel_directory $PACKAGESDIR $PACKAGENAME ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

	# ######### Create package directories #########
	#
	if [ ! -d $PACKAGESDIR ]
	then
		mkdir $PACKAGESDIR ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	fi
	if [ ! -d $PACKAGESDIR/$PACKAGENAME ]
	then
		mkdir $PACKAGESDIR/$PACKAGENAME ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	fi
	if [ ! -d $KIBANA_RSTDIR ]
	then
		mkdir $KIBANA_RSTDIR ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	fi
	if [ ! -d $KIBANA_RSTDIR/source ]
	then
		mkdir $KIBANA_RSTDIR/source ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	fi
	echo

	# ######### Delete documentation build directories #########
	#
	rm -rf $KIBANA_RSTDIR/build
	echo 

	# ######### Download git documentation repository #########
	#
	if [ ! -d $KIBANA_SOURCEDIR/.git ]
	then
		# ######### Clone git documentation repository #########
		#
		echo "--- `date +%T.%N | cut -c 1-12` - Clone git documentation repository -------------------------------------"
		echo
		cd $PACKAGESDIR; env GIT_SSL_NO_VERIFY=true git clone -b 3.0 https://github.com/elasticsearch/kibana.git/ $KIBANA_SOURCEDIR ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		echo
	else
		# Pull new revision from remote git documentation repository
		#
		echo "--- `date +%T.%N | cut -c 1-12` - Pull new revision from remote git documentation repository -------------"
		echo
		cd $KIBANA_SOURCEDIR; env GIT_SSL_NO_VERIFY=true git pull ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		echo
	fi

	# ######### Convert documentation from asciidoc to rst #########
	#
	convert_asciidoc_to_rst $KIBANA_SOURCEDIR/docs ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	cp -R $KIBANA_SOURCEDIR/docs/* $KIBANA_RSTDIR/source

	# ######### sphinx build html doc pages #########
	#
	build_sphinx_html $KIBANA_RSTDIR $KIBANA_DOCDIR ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

	# ######### Copy required files into new directory #########
	#
	echo "--- `date +%T.%N | cut -c 1-12` - Copy required files into new tmp directory -----------------------------"
	echo

	cp -R $KIBANA_RSTDIR/* $PACKAGESDIR/$PACKAGENAME ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	echo

	# ######### Create new gzip archiv #########
	#
	create_new_archive $PACKAGESDIR $PACKAGENAME ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

	# ######### Delete temp directory #########
	#
	delete_temp_devel_directory $PACKAGESDIR $PACKAGENAME ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

	return $RETVAL

}

# ######### Creation of a new elasticsearch documentation package #########
#
build_elasticsearchdoc_package(){

	local RETVAL=0
	local OLDVERSION=$(check_elasticsearch_status; echo $ES_VERSION | awk -F'.' '{print $1 "." $2 "." $3+0}')

	read -e -p "New version: " -i $OLDVERSION NEWVERSION
	echo

	local PACKAGENAME="elasticsearch-doc-$NEWVERSION"

	echo "--- `date +%T.%N | cut -c 1-12` - Creation of a new elasticsearch documentation packages -----------------"
	echo

	# ######### Check .netrc and .gitconfig user configuration for git client #########
	#
	check_netrc_config ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	check_git_config ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

	# ######### Delete temp directory #########
	#
	delete_temp_devel_directory $PACKAGESDIR $PACKAGENAME ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

	# ######### Create package directories #########
	#
	if [ ! -d $PACKAGESDIR ]
	then
		mkdir $PACKAGESDIR ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	fi
	if [ ! -d $PACKAGESDIR/$PACKAGENAME ]
	then
		mkdir $PACKAGESDIR/$PACKAGENAME ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	fi
	if [ ! -d $ELASTICSEARCH_RSTDIR ]
	then
		mkdir $ELASTICSEARCH_RSTDIR ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	fi
	if [ ! -d $ELASTICSEARCH_RSTDIR/source ]
	then
		mkdir $ELASTICSEARCH_RSTDIR/source ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	fi
	echo

	# ######### Delete documentation build directories #########
	#
	rm -rf $ELASTICSEARCH_RSTDIR/build
	echo 

	# ######### Download git documentation repository #########
	#
	if [ ! -d $ELASTICSEARCH_SOURCEDIR/.git ]
	then
		# ######### Clone git documentation repository #########
		#
		echo "--- `date +%T.%N | cut -c 1-12` - Clone git documentation repository -------------------------------------"
		echo
		cd $PACKAGESDIR; env GIT_SSL_NO_VERIFY=true git clone https://github.com/elasticsearch/elasticsearch.git/ $ELASTICSEARCH_SOURCEDIR ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		echo
	else
		# Pull new revision from remote git documentation repository
		#
		echo "--- `date +%T.%N | cut -c 1-12` - Pull new revision from remote git documentation repository -------------"
		echo
		cd $ELASTICSEARCH_SOURCEDIR; env GIT_SSL_NO_VERIFY=true git pull ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		echo
	fi

	# ######### Convert documentation from asciidoc to rst #########
	#
	convert_asciidoc_to_rst $ELASTICSEARCH_SOURCEDIR/docs/community ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	convert_asciidoc_to_rst $ELASTICSEARCH_SOURCEDIR/docs/groovy-api ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	convert_asciidoc_to_rst $ELASTICSEARCH_SOURCEDIR/docs/java-api ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	convert_asciidoc_to_rst $ELASTICSEARCH_SOURCEDIR/docs/javascript ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	convert_asciidoc_to_rst $ELASTICSEARCH_SOURCEDIR/docs/perl ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	convert_asciidoc_to_rst $ELASTICSEARCH_SOURCEDIR/docs/python ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	convert_asciidoc_to_rst $ELASTICSEARCH_SOURCEDIR/docs/reference ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	convert_asciidoc_to_rst $ELASTICSEARCH_SOURCEDIR/docs/resiliency ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	convert_asciidoc_to_rst $ELASTICSEARCH_SOURCEDIR/docs/river ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	convert_asciidoc_to_rst $ELASTICSEARCH_SOURCEDIR/docs/ruby ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

	cp -R $ELASTICSEARCH_SOURCEDIR/docs/* $ELASTICSEARCH_RSTDIR/source ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

	# ######### sphinx build html doc pages #########
	#
	build_sphinx_html $ELASTICSEARCH_RSTDIR $ELASTICSEARCH_DOCDIR ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

	# ######### Copy required files into new directory #########
	#
	echo "--- `date +%T.%N | cut -c 1-12` - Copy required files into new tmp directory -----------------------------"
	echo

	cp -R $ELASTICSEARCH_RSTDIR/* $PACKAGESDIR/$PACKAGENAME ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	echo

	# ######### Create new gzip archiv #########
	#
	create_new_archive $PACKAGESDIR $PACKAGENAME ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

	# ######### Delete temp directory #########
	#
	delete_temp_devel_directory $PACKAGESDIR $PACKAGENAME ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

	return $RETVAL

}


# ######### Creation of a new rsyslog documentation package #########
#
build_rsyslogdoc_package(){

	local RETVAL=0
	local OLDVERSION=$(check_rsyslog_status; echo $RSYSLOG_VERSION | awk -F'.' '{print $1 "." $2 "." $3+0}')

	read -e -p "New version: " -i $OLDVERSION NEWVERSION
	echo

	local PACKAGENAME="rsyslog-doc-$NEWVERSION"

	echo "--- `date +%T.%N | cut -c 1-12` - Creation of a new rsyslog documentation packages -----------------------"
	echo

	# ######### Check .netrc and .gitconfig user configuration for git client #########
	#
	check_netrc_config ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	check_git_config ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

	# ######### Delete temp directory #########
	#
	delete_temp_devel_directory $PACKAGESDIR $PACKAGENAME ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

	# ######### Create package directories #########
	#
	if [ ! -d $PACKAGESDIR ]
	then
		mkdir $PACKAGESDIR ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	fi
	if [ ! -d $PACKAGESDIR/$PACKAGENAME ]
	then
		mkdir $PACKAGESDIR/$PACKAGENAME ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	fi
	if [ ! -d $RSYSLOG_RSTDIR ]
	then
		mkdir $RSYSLOG_RSTDIR ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	fi
	echo

	# ######### Delete documentation build directories #########
	#
	rm -rf $RSYSLOG_RSTDIR/build
	echo 

	# ######### Download git documentation repository #########
	#
	if [ ! -d $RSYSLOG_RSTDIR/.git ]
	then
		# ######### Clone git documentation repository #########
		#
		echo "--- `date +%T.%N | cut -c 1-12` - Clone git documentation repository -------------------------------------"
		echo
		rm -R $RSYSLOG_RSTDIR
		cd $PACKAGESDIR; env GIT_SSL_NO_VERIFY=true git clone https://github.com/rsyslog/rsyslog-doc.git/ $RSYSLOG_RSTDIR ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		echo
	else
		# Pull new revision from remote git documentation repository
		#
		echo "--- `date +%T.%N | cut -c 1-12` - Pull new revision from remote git documentation repository -------------"
		echo
		cd $RSYSLOG_RSTDIR; env GIT_SSL_NO_VERIFY=true git pull ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		echo
	fi

	# ######### sphinx build html doc pages #########
	#
	build_sphinx_html $RSYSLOG_RSTDIR $RSYSLOG_DOCDIR ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

	# ######### Copy required files into new directory #########
	#
	echo "--- `date +%T.%N | cut -c 1-12` - Copy required files into new tmp directory -----------------------------"
	echo

	cp -R $RSYSLOG_RSTDIR/* $PACKAGESDIR/$PACKAGENAME ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	echo

	# ######### Create new gzip archiv #########
	#
	create_new_archive $PACKAGESDIR $PACKAGENAME ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

	# ######### Delete temp directory #########
	#
	delete_temp_devel_directory $PACKAGESDIR $PACKAGENAME ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

	return $RETVAL

}